import * as path from 'path';
import fse from 'fs-extra';

import * as Electron from 'electron';

import log from 'electron-log';

import { IpcMainApi, PlatformApi } from '@ucap/electron-common';

import { FileChannel } from '@ucap/electron-native';
import { FileUtil, BrowserWindowUtil } from '@ucap/electron-core';

export class AppFileService
  implements PlatformApi.AfterInit, PlatformApi.BeforeDestroy {
  static readonly ucapClassName: string = 'appFileService';

  ucapAfterInit(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  ucapBeforeDestroy(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  @IpcMainApi.Handle(FileChannel.save)
  async handleSave(
    event: Electron.IpcMainEvent,
    buffer: Buffer,
    fileName: string,
    mimeType: string,
    savePath?: string
  ): Promise<string> {
    log.debug('handleSave');

    let savedDir: string;
    let savedFileName: string;
    let savedPath: string;

    if (!!savePath) {
      fse.ensureDirSync(savePath);
      savedDir = savePath;
    } else {
      savedDir = Electron.app.getPath('downloads');
    }

    savedFileName = await FileUtil.uniqueFileName(
      path.join(savedDir, fileName)
    );

    savedPath = path.join(savedDir, savedFileName);

    fse.writeFileSync(savedPath, buffer);

    return savedPath;
  }

  @IpcMainApi.Handle(FileChannel.read)
  async handleRead(
    event: Electron.IpcMainEvent,
    path: string
  ): Promise<Buffer> {
    return fse.readFileSync(path);
  }

  @IpcMainApi.Handle(FileChannel.openFolder)
  async handleOpenFolder(
    event: Electron.IpcMainEvent,
    folderPath?: string,
    make?: boolean
  ): Promise<boolean> {
    log.debug('handleOpenFolder');
    let openDir: string;

    if (!!folderPath) {
      if (!fse.existsSync(folderPath)) {
        if (!make) {
          return false;
        }
        fse.ensureDirSync(folderPath);
      }
      openDir = folderPath;
    } else {
      openDir = Electron.app.getPath('downloads');
    }

    Electron.shell.openPath(openDir);

    return true;
  }

  @IpcMainApi.Handle(FileChannel.openItem)
  async handleOpenItem(
    event: Electron.IpcMainEvent,
    filePath: string
  ): Promise<boolean> {
    log.debug('handleOpenItem');

    if (!fse.existsSync(filePath)) {
      return false;
    }

    Electron.shell.openPath(filePath);
  }

  @IpcMainApi.Handle(FileChannel.path)
  async handlePath(
    event: Electron.IpcMainEvent,
    name: 'home' | 'documents' | 'downloads' | 'music' | 'pictures' | 'videos',
    appendPaths: string[]
  ): Promise<string> {
    log.debug('handlePath');

    let appPath = Electron.app.getPath(name);
    if (!!appendPaths && 0 < appendPaths.length) {
      appPath = path.join(appPath, ...appendPaths);
    }
    return appPath;
  }

  @IpcMainApi.Handle(FileChannel.selectForOpen)
  async handleSelectForOpen(
    event: Electron.IpcMainEvent,
    option: {
      title?: string;
      defaultPath?: string;
      filters?: { extensions: string[]; name: string }[];
      properties?: Array<
        | 'openFile'
        | 'openDirectory'
        | 'multiSelections'
        | 'showHiddenFiles'
        | 'createDirectory'
        | 'promptToCreate'
        | 'noResolveAliases'
        | 'treatPackageAsDirectory'
      >;
      message?: string;
    }
  ): Promise<string> {
    log.debug('handleSelectForOpen');

    const result = await Electron.dialog.showOpenDialog(
      BrowserWindowUtil.main(),
      option
    );

    if (result.canceled || !result.filePaths || 0 === result.filePaths.length) {
      return undefined;
    }

    return result.filePaths[0];
  }

  @IpcMainApi.Handle(FileChannel.selectForSave)
  async handleSelectForSave(
    event: Electron.IpcMainEvent,
    option: {
      title?: string;
      defaultPath?: string;
      filters?: { extensions: string[]; name: string }[];
      message?: string;
    }
  ): Promise<string> {
    log.debug('handleSelectForSave');

    const result = await Electron.dialog.showSaveDialog(
      BrowserWindowUtil.main(),
      option
    );

    if (result.canceled || !result.filePath) {
      return undefined;
    }

    return result.filePath;
  }
}
