import * as Electron from 'electron';

import AutoLaunch from 'auto-launch';

import {
  NotificationRequest,
  NotificationType,
  AppInitInfo
} from '@ucap/native';

import { AppChannel as ElectronAppChannel } from '@ucap/electron-core';
import { IpcMainApi, AppApi, PlatformApi } from '@ucap/electron-common';
import { AppChannel, ChatChannel, MessageChannel } from '@ucap/electron-native';
import { NotifyWindowService } from '@ucap/electron-notify-window';
import { BrowserWindowUtil } from '@ucap/electron-core';
import { ObjectUtil } from '@ucap/core';
import { AppI18nService } from './app-i18n.service';
import { AppLoggerService } from './app-logger.service';

export class AppService
  implements PlatformApi.AfterInit, PlatformApi.BeforeDestroy {
  static readonly ucapClassName: string = 'appService';

  appI18nService: AppI18nService;
  appLoggerService: AppLoggerService;

  private autoLaunch: AutoLaunch;
  private notifyWindowService: NotifyWindowService;
  private appInitInfo: AppInitInfo;
  private logined: boolean;
  private appTray: Electron.Tray;

  constructor(private configuration: AppApi.AppConfiguration) {
    this.autoLaunch = new AutoLaunch({ name: '' });
  }

  ucapAfterInit(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  ucapBeforeDestroy(): void | Promise<void> {
    return new Promise<void>(async (resolve, reject) => {
      try {
        resolve();
      } catch (error) {
        reject(error);
      }
    });
  }

  @AppApi.On(ElectronAppChannel.ready)
  onAppReady() {
    this.notifyWindowService = new NotifyWindowService(
      this.configuration?.notification?.option
    );

    this.notifyWindowService.options.defaultWindow.webPreferences.preload = this.configuration?.notification?.preloadPath;
    this.notifyWindowService.templatePath = this.configuration?.notification?.templatePath;

    this._setTray();
  }

  @IpcMainApi.Handle(AppChannel.version)
  async handleVersion(): Promise<string> {
    this.appLoggerService.debug('handleVersion');
    return Electron.app.getVersion();
  }

  @IpcMainApi.Handle(AppChannel.postInit)
  async handlePostInit(): Promise<AppInitInfo> {
    this.appLoggerService.debug('handlePostInit');
    const appInitInfo = ObjectUtil.deepClone(this.appInitInfo);
    this.appInitInfo = undefined;
    return appInitInfo;
  }

  @IpcMainApi.Handle(AppChannel.postLogin)
  async handlePostLogin(): Promise<void> {
    this.appLoggerService.debug('handlePostLogin');
    this.logined = true;
    this._setTray();
  }

  @IpcMainApi.Handle(AppChannel.postLogout)
  async handlePostLogout(): Promise<void> {
    this.appLoggerService.debug('handlePostLogout');
    this.logined = false;
    this._setTray();
  }

  @IpcMainApi.Handle(AppChannel.postDestroy)
  async handlePostDestroy(): Promise<void> {
    this.appLoggerService.debug('handlePostDestroy');
  }

  @IpcMainApi.Handle(AppChannel.changeAutoLaunch)
  async handleChangeAutoLaunch(
    event: Electron.IpcMainEvent,
    isAutoLaunch: boolean
  ): Promise<boolean> {
    this.appLoggerService.debug('handleChangeAutoLaunch');
    try {
      if (isAutoLaunch) {
        await this.autoLaunch.enable();
      } else {
        await this.autoLaunch.disable();
      }
    } catch (error) {
      return false;
    }
    return true;
  }

  @IpcMainApi.Handle(AppChannel.showNotify)
  async handleShowNotify(
    event: Electron.IpcMainEvent,
    req: NotificationRequest
  ): Promise<void> {
    this.appLoggerService.debug('handleShowNotify');
    const mainWindow = BrowserWindowUtil.main();

    this.notifyWindowService.notify({
      title: req.title,
      text: req.contents,
      image: req.image || this.configuration.notification.defaultImagePath,
      sound: req.useSound
        ? this.configuration.notification.defaultSoundPath
        : undefined,
      displayTime: req.displayTime,
      onClick: (event) => {
        if (mainWindow) {
          mainWindow.flashFrame(false);

          switch (req.type) {
            case NotificationType.Event:
              {
                mainWindow.webContents.send(ChatChannel.onOpen$, req.seq);
              }
              break;
            case NotificationType.Message:
              {
                mainWindow.webContents.send(MessageChannel.onOpen$, req.seq);
              }
              break;
          }

          mainWindow.show();
          event.close();
        }
      }
    });

    if (!mainWindow.isVisible()) {
      mainWindow.minimize();
    }

    mainWindow.flashFrame(true);
  }

  @IpcMainApi.Handle(AppChannel.closeAllNotify)
  async handleCloseAllNotify(): Promise<void> {
    this.appLoggerService.debug('handleCloseAllNotify');

    const mainWindow = BrowserWindowUtil.main();

    if (!!mainWindow) {
      mainWindow.flashFrame(false);
    }
    this.notifyWindowService.closeAll();
  }

  @IpcMainApi.Handle(AppChannel.checkForUpdates)
  async handleCheckForUpdates(): Promise<void> {
    this.appLoggerService.debug('handleCheckForUpdates');
  }

  @IpcMainApi.Handle(AppChannel.applyInstantUpdates)
  async handleApplyInstantUpdates(): Promise<void> {
    this.appLoggerService.debug('handleApplyInstantUpdates');
  }

  @IpcMainApi.Handle(AppChannel.startedCheckForUpdate)
  async handleStartedCheckForUpdate(): Promise<void> {
    this.appLoggerService.debug('handleStartedCheckForUpdate');
  }

  @IpcMainApi.Handle(AppChannel.startCheckForUpdate)
  async handleStartCheckForUpdate(): Promise<void> {
    this.appLoggerService.debug('handleStartCheckForUpdate');
  }

  @IpcMainApi.Handle(AppChannel.stopCheckForUpdate)
  async handleStopCheckForUpdate(): Promise<void> {
    this.appLoggerService.debug('handleStopCheckForUpdate');
  }

  @IpcMainApi.Handle(AppChannel.exit)
  async handleExit(): Promise<void> {
    this.appLoggerService.debug('handleExit');

    Electron.app.exit();
  }

  setInitInfo(appInitInfo: AppInitInfo) {
    if (!this.appInitInfo) {
      this.appInitInfo = {};
    }

    this.appInitInfo = {
      ...this.appInitInfo,
      ...appInitInfo
    };
  }

  private _setTray() {
    const trayMenu: Electron.MenuItemConstructorOptions[] = [];

    if (this.logined) {
      trayMenu.push(
        {
          label: this.appI18nService.t('electron:logout'),
          click: () => {
            const mainWindow = BrowserWindowUtil.main();
            if (!!mainWindow) {
              mainWindow.show();
              mainWindow.webContents.send(AppChannel.onLogout$);
            }
          }
        },
        {
          label: this.appI18nService.t('electron:settings'),
          click: () => {
            const mainWindow = BrowserWindowUtil.main();
            if (!!mainWindow) {
              mainWindow.show();
              mainWindow.webContents.send(AppChannel.onShowSetting$);
            }
          }
        }
      );
    }

    trayMenu.push(
      {
        label: this.appI18nService.t('electron:version'),
        submenu: [{ label: Electron.app.getVersion() }]
      },
      {
        label: this.appI18nService.t('electron:exit'),
        click: () => {
          const mainWindow = BrowserWindowUtil.main();
          if (!!mainWindow) {
            mainWindow.show();
            mainWindow.webContents.send(AppChannel.onExit$);
          }
        }
      }
    );

    const menu = Electron.Menu.buildFromTemplate(trayMenu);

    if (__DARWIN__) {
      Electron.app.dock.setMenu(menu);
    } else {
      if (!this.appTray) {
        const appIcon = this.configuration['assets']['appIcon'];
        this.appTray = new Electron.Tray(appIcon);
        this.appTray.setToolTip('M Messenger');
        this.appTray.on('click', () => {
          const mainWindow = BrowserWindowUtil.main();
          mainWindow.isVisible() ? mainWindow.hide() : mainWindow.show();
        });
      }

      this.appTray.setContextMenu(menu);
    }
  }
}
